unit QfelColorComboBox;

{}
{ (C)1997 by Marcin Qfel Zaleski                                               }
{ partions (C)1996 by Ray Konopka                                              }
{}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TQfelColorComboBox = class(TCustomComboBox)
    private
      FSelectedColor : TColor;
      function  GetSelectedColor:TColor;
      procedure SetSelectedColor(Value:TColor);
      procedure DrawItem(Index:Integer;Rect:TRect;State:TOwnerDrawState); override;
    public
      constructor Create(AOwner:TComponent); override;
      procedure AddColor(Color:TColor;ColorName:string);
      property SelectedColor:TColor read GetSelectedColor write SetSelectedColor;
    published
      property Color;
      property CTl3D;
      property DragCursor;
      property DragMode;
      property Enabled;
      property Font;
      property ItemHeight;
      property ParentColor;
      property ParentCtl3D;
      property ParentFont;
      property ParentShowHint;
      property PopupMenu;
      property ShowHint;
      property Sorted;
      property TabOrder;
      property TabStop;
      property Visible;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnDropDown;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Qfel', [TQfelColorComboBox]);
end;

constructor TQfelColorComboBox.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FSelectedColor := clNone;
end;

function TQfelColorComboBox.GetSelectedColor:TColor;
begin
  if ItemIndex=-1 then
    Result := clNone
  else
    Result := TColor(Items.Objects[ItemIndex]);
end;

procedure TQfelColorComboBox.SetSelectedColor(Value:TColor);
var
  I : Integer;
begin
  I := 0;
  while ((I<Items.Count)and(Value<>TColor(Items.Objects[I]))) do
    Inc(I);
  if I=Items.Count then
    ItemIndex := -1
  else
    ItemIndex := I;
end;

procedure TQfelColorComboBox.AddColor(Color:TColor;ColorName:string);
begin
  Items.AddObject(ColorName,TObject(Color));
end;

procedure TQfelColorComboBox.DrawItem(Index:Integer;Rect:TRect;State:TOwnerDrawState);
var
  BackColor : TColor;
begin
  with Canvas do
    begin
      //czycimy to
      FillRect(Rect);
      //zapamituje to
      BackColor := Brush.Color;
      //kolory systemowe zalene od stanu kontrolki
      if odDisabled in State then
        begin
          Pen.Color := clGrayText;
          Font.Color := clGrayText;
        end
      else if odFocused in State then
        begin
          Pen.Color := clHighlightText;
          Font.Color := clHighlightText;
        end
      else
        begin
          Pen.Color := clBtnText;
          Font.Color := clBtnText;
        end;
      //rysujemy
      Brush.Color := TColor(Items.Objects[Index]);
      Rectangle(Rect.Left+2,Rect.Top+2,Rect.Left+22,Rect.Bottom-2);
      //piszemy
      Inc(Rect.Left,26);
      Brush.Color := BackColor;
      DrawText(Handle,PChar(Items[Index]),-1,Rect,DT_LEFT or DT_NOCLIP or
                                    DT_NOPREFIX or DT_SINGLELINE or DT_VCENTER);
    end;
end;

end.
